import 'package:flutter/cupertino.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/datetime/nui_dt_util.dart';
import 'package:nui_core/nui_core.dart';

typedef CustomStylingByColRow = NUITableCellStyle Function(int row, int column);
typedef CustomHeaderStyling = NUITableCellStyle Function(int row, int column);

class NUITableRow{
  NUITableRow(this.cells);
  List<NUITableCell> cells;
  int col;

  static NUITableRow headersFromEntity(NUIEntMapper mapper){
    if(mapper.fields().isNullOrEmpty()) return null;

    final cells = List<NUITableCell>();
    for(NUIEntField field in mapper.fields()){
      if(field.type == NUIEntType.STRING || field.type == NUIEntType.DOUBLE || field.type == NUIEntType.DATE || field.type == NUIEntType.INTEGER || field.type == NUIEntType.BOOLEAN) {
        cells.add(NUITableCell(value: field.name.toUpperCase(), type: NUIEntType.STRING));
      }
    }
    return NUITableRow(cells);
  }

  static NUITableRow fromEntity(NUIEnt entity){
    if(entity.mapper().fields().isNullOrEmpty()) return null;

    final cells = List<NUITableCell>();
    for(NUIEntField field in entity.mapper().fields()){
      if(field.type == NUIEntType.STRING || field.type == NUIEntType.DOUBLE || field.type == NUIEntType.DATE || field.type == NUIEntType.INTEGER || field.type == NUIEntType.BOOLEAN) {
        print("Entity field name : ${field.name}");
        cells.add(NUITableCell(value: field.runGetter(entity), type: field.type));
      }
    }
    print("Number of cells is : ${cells.length}");
    return NUITableRow(cells);
  }
}

class NUITableCell{
  NUITableCell({@required this.value, this.type: NUIEntType.STRING, this.style, this.formatting});
  dynamic value;
  NUIEntType type;
  NUITableCellStyle style;
  String formatting;
  int row;
  int col;

  String getFormattedValue(){
    if(type == NUIEntType.DATE && value is DateTime){
      if(isNullOrEmpty(formatting)){
        return dateToString(value, format: NUIDTUtil.FORMAT_dd_MMM_yyyy_hh_mm_a);
      }
      else{
        return dateToString(value, format: formatting);
      }
    }
    else if(type == NUIEntType.DOUBLE && (value is int || value is double)){
      switch(formatting){
        case NUIFormatting.ONE_DECIMAL: return formatNumber(value, showComma: true, decimals: 1);
        case NUIFormatting.TWO_DECIMAL: return formatNumber(value, showComma: true, decimals: 2);
        case NUIFormatting.ROUNDED: return formatNumber(value, showComma: true, decimals: 0);
      }
    }
    return value.toString();
  }
}

class NUITableCellStyle{
  NUITableCellStyle({this.alignment, this.textStyle, this.background, this.width, this.height, this.outlineColor, this.outlineThickness, this.padding});
  TextStyle textStyle;
  Color background;
  double width;
  double height;
  Color outlineColor;
  double outlineThickness;
  Alignment alignment;
  double padding;
}